"""Generated client library for assuredworkloads version v1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.assuredworkloads.v1 import assuredworkloads_v1_messages as messages


class AssuredworkloadsV1(base_api.BaseApiClient):
  """Generated client library for service assuredworkloads version v1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://assuredworkloads.googleapis.com/'
  MTLS_BASE_URL = 'https://assuredworkloads.mtls.googleapis.com/'

  _PACKAGE = 'assuredworkloads'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'AssuredworkloadsV1'
  _URL_VERSION = 'v1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new assuredworkloads handle."""
    url = url or self.BASE_URL
    super(AssuredworkloadsV1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.organizations_locations_operations = self.OrganizationsLocationsOperationsService(self)
    self.organizations_locations_workloads_violations = self.OrganizationsLocationsWorkloadsViolationsService(self)
    self.organizations_locations_workloads = self.OrganizationsLocationsWorkloadsService(self)
    self.organizations_locations = self.OrganizationsLocationsService(self)
    self.organizations = self.OrganizationsService(self)

  class OrganizationsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the organizations_locations_operations resource."""

    _NAME = 'organizations_locations_operations'

    def __init__(self, client):
      super(AssuredworkloadsV1.OrganizationsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (AssuredworkloadsOrganizationsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='assuredworkloads.organizations.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='AssuredworkloadsOrganizationsLocationsOperationsGetRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`. NOTE: the `name` binding allows API services to override the binding to use different resource name schemes, such as `users/*/operations`. To override the binding, API services can add a binding such as `"/v1/{name=users/*}/operations"` to their service configuration. For backwards compatibility, the default name includes the operations collection id, however overriding users must ensure the name binding is the parent resource, without the operations collection id.

      Args:
        request: (AssuredworkloadsOrganizationsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='assuredworkloads.organizations.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+name}/operations',
        request_field='',
        request_type_name='AssuredworkloadsOrganizationsLocationsOperationsListRequest',
        response_type_name='GoogleLongrunningListOperationsResponse',
        supports_download=False,
    )

  class OrganizationsLocationsWorkloadsViolationsService(base_api.BaseApiService):
    """Service class for the organizations_locations_workloads_violations resource."""

    _NAME = 'organizations_locations_workloads_violations'

    def __init__(self, client):
      super(AssuredworkloadsV1.OrganizationsLocationsWorkloadsViolationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Acknowledge(self, request, global_params=None):
      r"""Acknowledges an existing violation. By acknowledging a violation, users acknowledge the existence of a compliance violation in their workload and decide to ignore it due to a valid business justification. Acknowledgement is a permanent operation and it cannot be reverted.

      Args:
        request: (AssuredworkloadsOrganizationsLocationsWorkloadsViolationsAcknowledgeRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudAssuredworkloadsV1AcknowledgeViolationResponse) The response message.
      """
      config = self.GetMethodConfig('Acknowledge')
      return self._RunMethod(
          config, request, global_params=global_params)

    Acknowledge.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/workloads/{workloadsId}/violations/{violationsId}:acknowledge',
        http_method='POST',
        method_id='assuredworkloads.organizations.locations.workloads.violations.acknowledge',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:acknowledge',
        request_field='googleCloudAssuredworkloadsV1AcknowledgeViolationRequest',
        request_type_name='AssuredworkloadsOrganizationsLocationsWorkloadsViolationsAcknowledgeRequest',
        response_type_name='GoogleCloudAssuredworkloadsV1AcknowledgeViolationResponse',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Retrieves Assured Workload Violation based on ID.

      Args:
        request: (AssuredworkloadsOrganizationsLocationsWorkloadsViolationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudAssuredworkloadsV1Violation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/workloads/{workloadsId}/violations/{violationsId}',
        http_method='GET',
        method_id='assuredworkloads.organizations.locations.workloads.violations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='AssuredworkloadsOrganizationsLocationsWorkloadsViolationsGetRequest',
        response_type_name='GoogleCloudAssuredworkloadsV1Violation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists the Violations in the AssuredWorkload Environment. Callers may also choose to read across multiple Workloads as per [AIP-159](https://google.aip.dev/159) by using '-' (the hyphen or dash character) as a wildcard character instead of workload-id in the parent. Format `organizations/{org_id}/locations/{location}/workloads/-`.

      Args:
        request: (AssuredworkloadsOrganizationsLocationsWorkloadsViolationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudAssuredworkloadsV1ListViolationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/workloads/{workloadsId}/violations',
        http_method='GET',
        method_id='assuredworkloads.organizations.locations.workloads.violations.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'interval_endTime', 'interval_startTime', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/violations',
        request_field='',
        request_type_name='AssuredworkloadsOrganizationsLocationsWorkloadsViolationsListRequest',
        response_type_name='GoogleCloudAssuredworkloadsV1ListViolationsResponse',
        supports_download=False,
    )

  class OrganizationsLocationsWorkloadsService(base_api.BaseApiService):
    """Service class for the organizations_locations_workloads resource."""

    _NAME = 'organizations_locations_workloads'

    def __init__(self, client):
      super(AssuredworkloadsV1.OrganizationsLocationsWorkloadsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates Assured Workload.

      Args:
        request: (AssuredworkloadsOrganizationsLocationsWorkloadsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/workloads',
        http_method='POST',
        method_id='assuredworkloads.organizations.locations.workloads.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['externalId'],
        relative_path='v1/{+parent}/workloads',
        request_field='googleCloudAssuredworkloadsV1Workload',
        request_type_name='AssuredworkloadsOrganizationsLocationsWorkloadsCreateRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes the workload. Make sure that workload's direct children are already in a deleted state, otherwise the request will fail with a FAILED_PRECONDITION error.

      Args:
        request: (AssuredworkloadsOrganizationsLocationsWorkloadsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/workloads/{workloadsId}',
        http_method='DELETE',
        method_id='assuredworkloads.organizations.locations.workloads.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['etag'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='AssuredworkloadsOrganizationsLocationsWorkloadsDeleteRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets Assured Workload associated with a CRM Node.

      Args:
        request: (AssuredworkloadsOrganizationsLocationsWorkloadsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudAssuredworkloadsV1Workload) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/workloads/{workloadsId}',
        http_method='GET',
        method_id='assuredworkloads.organizations.locations.workloads.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='AssuredworkloadsOrganizationsLocationsWorkloadsGetRequest',
        response_type_name='GoogleCloudAssuredworkloadsV1Workload',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Assured Workloads under a CRM Node.

      Args:
        request: (AssuredworkloadsOrganizationsLocationsWorkloadsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudAssuredworkloadsV1ListWorkloadsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/workloads',
        http_method='GET',
        method_id='assuredworkloads.organizations.locations.workloads.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/workloads',
        request_field='',
        request_type_name='AssuredworkloadsOrganizationsLocationsWorkloadsListRequest',
        response_type_name='GoogleCloudAssuredworkloadsV1ListWorkloadsResponse',
        supports_download=False,
    )

    def MutatePartnerPermissions(self, request, global_params=None):
      r"""Update the permissions settings for an existing partner workload. For force updates don't set etag field in the Workload. Only one update operation per workload can be in progress.

      Args:
        request: (AssuredworkloadsOrganizationsLocationsWorkloadsMutatePartnerPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudAssuredworkloadsV1Workload) The response message.
      """
      config = self.GetMethodConfig('MutatePartnerPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    MutatePartnerPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/workloads/{workloadsId}:mutatePartnerPermissions',
        http_method='PATCH',
        method_id='assuredworkloads.organizations.locations.workloads.mutatePartnerPermissions',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:mutatePartnerPermissions',
        request_field='googleCloudAssuredworkloadsV1MutatePartnerPermissionsRequest',
        request_type_name='AssuredworkloadsOrganizationsLocationsWorkloadsMutatePartnerPermissionsRequest',
        response_type_name='GoogleCloudAssuredworkloadsV1Workload',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates an existing workload. Currently allows updating of workload display_name and labels. For force updates don't set etag field in the Workload. Only one update operation per workload can be in progress.

      Args:
        request: (AssuredworkloadsOrganizationsLocationsWorkloadsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudAssuredworkloadsV1Workload) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/workloads/{workloadsId}',
        http_method='PATCH',
        method_id='assuredworkloads.organizations.locations.workloads.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='googleCloudAssuredworkloadsV1Workload',
        request_type_name='AssuredworkloadsOrganizationsLocationsWorkloadsPatchRequest',
        response_type_name='GoogleCloudAssuredworkloadsV1Workload',
        supports_download=False,
    )

    def RestrictAllowedResources(self, request, global_params=None):
      r"""Restrict the list of resources allowed in the Workload environment. The current list of allowed products can be found at https://cloud.google.com/assured-workloads/docs/supported-products In addition to assuredworkloads.workload.update permission, the user should also have orgpolicy.policy.set permission on the folder resource to use this functionality.

      Args:
        request: (AssuredworkloadsOrganizationsLocationsWorkloadsRestrictAllowedResourcesRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudAssuredworkloadsV1RestrictAllowedResourcesResponse) The response message.
      """
      config = self.GetMethodConfig('RestrictAllowedResources')
      return self._RunMethod(
          config, request, global_params=global_params)

    RestrictAllowedResources.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/workloads/{workloadsId}:restrictAllowedResources',
        http_method='POST',
        method_id='assuredworkloads.organizations.locations.workloads.restrictAllowedResources',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:restrictAllowedResources',
        request_field='googleCloudAssuredworkloadsV1RestrictAllowedResourcesRequest',
        request_type_name='AssuredworkloadsOrganizationsLocationsWorkloadsRestrictAllowedResourcesRequest',
        response_type_name='GoogleCloudAssuredworkloadsV1RestrictAllowedResourcesResponse',
        supports_download=False,
    )

  class OrganizationsLocationsService(base_api.BaseApiService):
    """Service class for the organizations_locations resource."""

    _NAME = 'organizations_locations'

    def __init__(self, client):
      super(AssuredworkloadsV1.OrganizationsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

  class OrganizationsService(base_api.BaseApiService):
    """Service class for the organizations resource."""

    _NAME = 'organizations'

    def __init__(self, client):
      super(AssuredworkloadsV1.OrganizationsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google/cloud/appengine_v1/proto/app_yaml.proto

from cloudsdk.google.protobuf.internal import enum_type_wrapper
from cloudsdk.google.protobuf import descriptor as _descriptor
from cloudsdk.google.protobuf import message as _message
from cloudsdk.google.protobuf import reflection as _reflection
from cloudsdk.google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cloudsdk.google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='google/cloud/appengine_v1/proto/app_yaml.proto',
  package='google.appengine.v1',
  syntax='proto3',
  serialized_options=b'\n\027com.google.appengine.v1B\014AppYamlProtoP\001Z<google.golang.org/genproto/googleapis/appengine/v1;appengine',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n.google/cloud/appengine_v1/proto/app_yaml.proto\x12\x13google.appengine.v1\x1a\x1egoogle/protobuf/duration.proto\x1a\x1cgoogle/api/annotations.proto\"\xe0\x01\n\x10\x41piConfigHandler\x12=\n\x10\x61uth_fail_action\x18\x01 \x01(\x0e\x32#.google.appengine.v1.AuthFailAction\x12\x34\n\x05login\x18\x02 \x01(\x0e\x32%.google.appengine.v1.LoginRequirement\x12\x0e\n\x06script\x18\x03 \x01(\t\x12:\n\x0esecurity_level\x18\x04 \x01(\x0e\x32\".google.appengine.v1.SecurityLevel\x12\x0b\n\x03url\x18\x05 \x01(\t\"\x8f\x02\n\x0c\x45rrorHandler\x12?\n\nerror_code\x18\x01 \x01(\x0e\x32+.google.appengine.v1.ErrorHandler.ErrorCode\x12\x13\n\x0bstatic_file\x18\x02 \x01(\t\x12\x11\n\tmime_type\x18\x03 \x01(\t\"\x95\x01\n\tErrorCode\x12\x1a\n\x16\x45RROR_CODE_UNSPECIFIED\x10\x00\x12\x16\n\x12\x45RROR_CODE_DEFAULT\x10\x00\x12\x19\n\x15\x45RROR_CODE_OVER_QUOTA\x10\x01\x12\x1d\n\x19\x45RROR_CODE_DOS_API_DENIAL\x10\x02\x12\x16\n\x12\x45RROR_CODE_TIMEOUT\x10\x03\x1a\x02\x10\x01\"\xcd\x05\n\x06UrlMap\x12\x11\n\turl_regex\x18\x01 \x01(\t\x12?\n\x0cstatic_files\x18\x02 \x01(\x0b\x32\'.google.appengine.v1.StaticFilesHandlerH\x00\x12\x34\n\x06script\x18\x03 \x01(\x0b\x32\".google.appengine.v1.ScriptHandlerH\x00\x12?\n\x0c\x61pi_endpoint\x18\x04 \x01(\x0b\x32\'.google.appengine.v1.ApiEndpointHandlerH\x00\x12:\n\x0esecurity_level\x18\x05 \x01(\x0e\x32\".google.appengine.v1.SecurityLevel\x12\x34\n\x05login\x18\x06 \x01(\x0e\x32%.google.appengine.v1.LoginRequirement\x12=\n\x10\x61uth_fail_action\x18\x07 \x01(\x0e\x32#.google.appengine.v1.AuthFailAction\x12Y\n\x1bredirect_http_response_code\x18\x08 \x01(\x0e\x32\x34.google.appengine.v1.UrlMap.RedirectHttpResponseCode\"\xdb\x01\n\x18RedirectHttpResponseCode\x12+\n\'REDIRECT_HTTP_RESPONSE_CODE_UNSPECIFIED\x10\x00\x12#\n\x1fREDIRECT_HTTP_RESPONSE_CODE_301\x10\x01\x12#\n\x1fREDIRECT_HTTP_RESPONSE_CODE_302\x10\x02\x12#\n\x1fREDIRECT_HTTP_RESPONSE_CODE_303\x10\x03\x12#\n\x1fREDIRECT_HTTP_RESPONSE_CODE_307\x10\x04\x42\x0e\n\x0chandler_type\"\xc0\x02\n\x12StaticFilesHandler\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x19\n\x11upload_path_regex\x18\x02 \x01(\t\x12N\n\x0chttp_headers\x18\x03 \x03(\x0b\x32\x38.google.appengine.v1.StaticFilesHandler.HttpHeadersEntry\x12\x11\n\tmime_type\x18\x04 \x01(\t\x12-\n\nexpiration\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x1d\n\x15require_matching_file\x18\x06 \x01(\x08\x12\x1c\n\x14\x61pplication_readable\x18\x07 \x01(\x08\x1a\x32\n\x10HttpHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"$\n\rScriptHandler\x12\x13\n\x0bscript_path\x18\x01 \x01(\t\")\n\x12\x41piEndpointHandler\x12\x13\n\x0bscript_path\x18\x01 \x01(\t\"\xeb\x01\n\x0bHealthCheck\x12\x1c\n\x14\x64isable_health_check\x18\x01 \x01(\x08\x12\x0c\n\x04host\x18\x02 \x01(\t\x12\x19\n\x11healthy_threshold\x18\x03 \x01(\r\x12\x1b\n\x13unhealthy_threshold\x18\x04 \x01(\r\x12\x19\n\x11restart_threshold\x18\x05 \x01(\r\x12\x31\n\x0e\x63heck_interval\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12*\n\x07timeout\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\"\xf7\x01\n\x0eReadinessCheck\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0c\n\x04host\x18\x02 \x01(\t\x12\x19\n\x11\x66\x61ilure_threshold\x18\x03 \x01(\r\x12\x19\n\x11success_threshold\x18\x04 \x01(\r\x12\x31\n\x0e\x63heck_interval\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12*\n\x07timeout\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x34\n\x11\x61pp_start_timeout\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\"\xf2\x01\n\rLivenessCheck\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0c\n\x04host\x18\x02 \x01(\t\x12\x19\n\x11\x66\x61ilure_threshold\x18\x03 \x01(\r\x12\x19\n\x11success_threshold\x18\x04 \x01(\r\x12\x31\n\x0e\x63heck_interval\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12*\n\x07timeout\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x30\n\rinitial_delay\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\"(\n\x07Library\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t*t\n\x0e\x41uthFailAction\x12 \n\x1c\x41UTH_FAIL_ACTION_UNSPECIFIED\x10\x00\x12\x1d\n\x19\x41UTH_FAIL_ACTION_REDIRECT\x10\x01\x12!\n\x1d\x41UTH_FAIL_ACTION_UNAUTHORIZED\x10\x02*b\n\x10LoginRequirement\x12\x15\n\x11LOGIN_UNSPECIFIED\x10\x00\x12\x12\n\x0eLOGIN_OPTIONAL\x10\x01\x12\x0f\n\x0bLOGIN_ADMIN\x10\x02\x12\x12\n\x0eLOGIN_REQUIRED\x10\x03*y\n\rSecurityLevel\x12\x16\n\x12SECURE_UNSPECIFIED\x10\x00\x12\x12\n\x0eSECURE_DEFAULT\x10\x00\x12\x10\n\x0cSECURE_NEVER\x10\x01\x12\x13\n\x0fSECURE_OPTIONAL\x10\x02\x12\x11\n\rSECURE_ALWAYS\x10\x03\x1a\x02\x10\x01\x42g\n\x17\x63om.google.appengine.v1B\x0c\x41ppYamlProtoP\x01Z<google.golang.org/genproto/googleapis/appengine/v1;appengineb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,])

_AUTHFAILACTION = _descriptor.EnumDescriptor(
  name='AuthFailAction',
  full_name='google.appengine.v1.AuthFailAction',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='AUTH_FAIL_ACTION_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AUTH_FAIL_ACTION_REDIRECT', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AUTH_FAIL_ACTION_UNAUTHORIZED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2533,
  serialized_end=2649,
)
_sym_db.RegisterEnumDescriptor(_AUTHFAILACTION)

AuthFailAction = enum_type_wrapper.EnumTypeWrapper(_AUTHFAILACTION)
_LOGINREQUIREMENT = _descriptor.EnumDescriptor(
  name='LoginRequirement',
  full_name='google.appengine.v1.LoginRequirement',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LOGIN_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGIN_OPTIONAL', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGIN_ADMIN', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGIN_REQUIRED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2651,
  serialized_end=2749,
)
_sym_db.RegisterEnumDescriptor(_LOGINREQUIREMENT)

LoginRequirement = enum_type_wrapper.EnumTypeWrapper(_LOGINREQUIREMENT)
_SECURITYLEVEL = _descriptor.EnumDescriptor(
  name='SecurityLevel',
  full_name='google.appengine.v1.SecurityLevel',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SECURE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SECURE_DEFAULT', index=1, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SECURE_NEVER', index=2, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SECURE_OPTIONAL', index=3, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SECURE_ALWAYS', index=4, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=b'\020\001',
  serialized_start=2751,
  serialized_end=2872,
)
_sym_db.RegisterEnumDescriptor(_SECURITYLEVEL)

SecurityLevel = enum_type_wrapper.EnumTypeWrapper(_SECURITYLEVEL)
AUTH_FAIL_ACTION_UNSPECIFIED = 0
AUTH_FAIL_ACTION_REDIRECT = 1
AUTH_FAIL_ACTION_UNAUTHORIZED = 2
LOGIN_UNSPECIFIED = 0
LOGIN_OPTIONAL = 1
LOGIN_ADMIN = 2
LOGIN_REQUIRED = 3
SECURE_UNSPECIFIED = 0
SECURE_DEFAULT = 0
SECURE_NEVER = 1
SECURE_OPTIONAL = 2
SECURE_ALWAYS = 3


_ERRORHANDLER_ERRORCODE = _descriptor.EnumDescriptor(
  name='ErrorCode',
  full_name='google.appengine.v1.ErrorHandler.ErrorCode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ERROR_CODE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR_CODE_DEFAULT', index=1, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR_CODE_OVER_QUOTA', index=2, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR_CODE_DOS_API_DENIAL', index=3, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR_CODE_TIMEOUT', index=4, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=b'\020\001',
  serialized_start=483,
  serialized_end=632,
)
_sym_db.RegisterEnumDescriptor(_ERRORHANDLER_ERRORCODE)

_URLMAP_REDIRECTHTTPRESPONSECODE = _descriptor.EnumDescriptor(
  name='RedirectHttpResponseCode',
  full_name='google.appengine.v1.UrlMap.RedirectHttpResponseCode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='REDIRECT_HTTP_RESPONSE_CODE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REDIRECT_HTTP_RESPONSE_CODE_301', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REDIRECT_HTTP_RESPONSE_CODE_302', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REDIRECT_HTTP_RESPONSE_CODE_303', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REDIRECT_HTTP_RESPONSE_CODE_307', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1117,
  serialized_end=1336,
)
_sym_db.RegisterEnumDescriptor(_URLMAP_REDIRECTHTTPRESPONSECODE)


_APICONFIGHANDLER = _descriptor.Descriptor(
  name='ApiConfigHandler',
  full_name='google.appengine.v1.ApiConfigHandler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='auth_fail_action', full_name='google.appengine.v1.ApiConfigHandler.auth_fail_action', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='login', full_name='google.appengine.v1.ApiConfigHandler.login', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='script', full_name='google.appengine.v1.ApiConfigHandler.script', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='security_level', full_name='google.appengine.v1.ApiConfigHandler.security_level', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='url', full_name='google.appengine.v1.ApiConfigHandler.url', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=134,
  serialized_end=358,
)


_ERRORHANDLER = _descriptor.Descriptor(
  name='ErrorHandler',
  full_name='google.appengine.v1.ErrorHandler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='error_code', full_name='google.appengine.v1.ErrorHandler.error_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='static_file', full_name='google.appengine.v1.ErrorHandler.static_file', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mime_type', full_name='google.appengine.v1.ErrorHandler.mime_type', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ERRORHANDLER_ERRORCODE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=361,
  serialized_end=632,
)


_URLMAP = _descriptor.Descriptor(
  name='UrlMap',
  full_name='google.appengine.v1.UrlMap',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='url_regex', full_name='google.appengine.v1.UrlMap.url_regex', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='static_files', full_name='google.appengine.v1.UrlMap.static_files', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='script', full_name='google.appengine.v1.UrlMap.script', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='api_endpoint', full_name='google.appengine.v1.UrlMap.api_endpoint', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='security_level', full_name='google.appengine.v1.UrlMap.security_level', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='login', full_name='google.appengine.v1.UrlMap.login', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='auth_fail_action', full_name='google.appengine.v1.UrlMap.auth_fail_action', index=6,
      number=7, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='redirect_http_response_code', full_name='google.appengine.v1.UrlMap.redirect_http_response_code', index=7,
      number=8, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _URLMAP_REDIRECTHTTPRESPONSECODE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='handler_type', full_name='google.appengine.v1.UrlMap.handler_type',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=635,
  serialized_end=1352,
)


_STATICFILESHANDLER_HTTPHEADERSENTRY = _descriptor.Descriptor(
  name='HttpHeadersEntry',
  full_name='google.appengine.v1.StaticFilesHandler.HttpHeadersEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='google.appengine.v1.StaticFilesHandler.HttpHeadersEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='google.appengine.v1.StaticFilesHandler.HttpHeadersEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1625,
  serialized_end=1675,
)

_STATICFILESHANDLER = _descriptor.Descriptor(
  name='StaticFilesHandler',
  full_name='google.appengine.v1.StaticFilesHandler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='google.appengine.v1.StaticFilesHandler.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='upload_path_regex', full_name='google.appengine.v1.StaticFilesHandler.upload_path_regex', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='http_headers', full_name='google.appengine.v1.StaticFilesHandler.http_headers', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mime_type', full_name='google.appengine.v1.StaticFilesHandler.mime_type', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expiration', full_name='google.appengine.v1.StaticFilesHandler.expiration', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='require_matching_file', full_name='google.appengine.v1.StaticFilesHandler.require_matching_file', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='application_readable', full_name='google.appengine.v1.StaticFilesHandler.application_readable', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_STATICFILESHANDLER_HTTPHEADERSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1355,
  serialized_end=1675,
)


_SCRIPTHANDLER = _descriptor.Descriptor(
  name='ScriptHandler',
  full_name='google.appengine.v1.ScriptHandler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='script_path', full_name='google.appengine.v1.ScriptHandler.script_path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1677,
  serialized_end=1713,
)


_APIENDPOINTHANDLER = _descriptor.Descriptor(
  name='ApiEndpointHandler',
  full_name='google.appengine.v1.ApiEndpointHandler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='script_path', full_name='google.appengine.v1.ApiEndpointHandler.script_path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1715,
  serialized_end=1756,
)


_HEALTHCHECK = _descriptor.Descriptor(
  name='HealthCheck',
  full_name='google.appengine.v1.HealthCheck',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='disable_health_check', full_name='google.appengine.v1.HealthCheck.disable_health_check', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host', full_name='google.appengine.v1.HealthCheck.host', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='healthy_threshold', full_name='google.appengine.v1.HealthCheck.healthy_threshold', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unhealthy_threshold', full_name='google.appengine.v1.HealthCheck.unhealthy_threshold', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='restart_threshold', full_name='google.appengine.v1.HealthCheck.restart_threshold', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='check_interval', full_name='google.appengine.v1.HealthCheck.check_interval', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timeout', full_name='google.appengine.v1.HealthCheck.timeout', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1759,
  serialized_end=1994,
)


_READINESSCHECK = _descriptor.Descriptor(
  name='ReadinessCheck',
  full_name='google.appengine.v1.ReadinessCheck',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='google.appengine.v1.ReadinessCheck.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host', full_name='google.appengine.v1.ReadinessCheck.host', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='failure_threshold', full_name='google.appengine.v1.ReadinessCheck.failure_threshold', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='success_threshold', full_name='google.appengine.v1.ReadinessCheck.success_threshold', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='check_interval', full_name='google.appengine.v1.ReadinessCheck.check_interval', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timeout', full_name='google.appengine.v1.ReadinessCheck.timeout', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='app_start_timeout', full_name='google.appengine.v1.ReadinessCheck.app_start_timeout', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1997,
  serialized_end=2244,
)


_LIVENESSCHECK = _descriptor.Descriptor(
  name='LivenessCheck',
  full_name='google.appengine.v1.LivenessCheck',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='google.appengine.v1.LivenessCheck.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host', full_name='google.appengine.v1.LivenessCheck.host', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='failure_threshold', full_name='google.appengine.v1.LivenessCheck.failure_threshold', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='success_threshold', full_name='google.appengine.v1.LivenessCheck.success_threshold', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='check_interval', full_name='google.appengine.v1.LivenessCheck.check_interval', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timeout', full_name='google.appengine.v1.LivenessCheck.timeout', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='initial_delay', full_name='google.appengine.v1.LivenessCheck.initial_delay', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2247,
  serialized_end=2489,
)


_LIBRARY = _descriptor.Descriptor(
  name='Library',
  full_name='google.appengine.v1.Library',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='google.appengine.v1.Library.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='version', full_name='google.appengine.v1.Library.version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2491,
  serialized_end=2531,
)

_APICONFIGHANDLER.fields_by_name['auth_fail_action'].enum_type = _AUTHFAILACTION
_APICONFIGHANDLER.fields_by_name['login'].enum_type = _LOGINREQUIREMENT
_APICONFIGHANDLER.fields_by_name['security_level'].enum_type = _SECURITYLEVEL
_ERRORHANDLER.fields_by_name['error_code'].enum_type = _ERRORHANDLER_ERRORCODE
_ERRORHANDLER_ERRORCODE.containing_type = _ERRORHANDLER
_URLMAP.fields_by_name['static_files'].message_type = _STATICFILESHANDLER
_URLMAP.fields_by_name['script'].message_type = _SCRIPTHANDLER
_URLMAP.fields_by_name['api_endpoint'].message_type = _APIENDPOINTHANDLER
_URLMAP.fields_by_name['security_level'].enum_type = _SECURITYLEVEL
_URLMAP.fields_by_name['login'].enum_type = _LOGINREQUIREMENT
_URLMAP.fields_by_name['auth_fail_action'].enum_type = _AUTHFAILACTION
_URLMAP.fields_by_name['redirect_http_response_code'].enum_type = _URLMAP_REDIRECTHTTPRESPONSECODE
_URLMAP_REDIRECTHTTPRESPONSECODE.containing_type = _URLMAP
_URLMAP.oneofs_by_name['handler_type'].fields.append(
  _URLMAP.fields_by_name['static_files'])
_URLMAP.fields_by_name['static_files'].containing_oneof = _URLMAP.oneofs_by_name['handler_type']
_URLMAP.oneofs_by_name['handler_type'].fields.append(
  _URLMAP.fields_by_name['script'])
_URLMAP.fields_by_name['script'].containing_oneof = _URLMAP.oneofs_by_name['handler_type']
_URLMAP.oneofs_by_name['handler_type'].fields.append(
  _URLMAP.fields_by_name['api_endpoint'])
_URLMAP.fields_by_name['api_endpoint'].containing_oneof = _URLMAP.oneofs_by_name['handler_type']
_STATICFILESHANDLER_HTTPHEADERSENTRY.containing_type = _STATICFILESHANDLER
_STATICFILESHANDLER.fields_by_name['http_headers'].message_type = _STATICFILESHANDLER_HTTPHEADERSENTRY
_STATICFILESHANDLER.fields_by_name['expiration'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_HEALTHCHECK.fields_by_name['check_interval'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_HEALTHCHECK.fields_by_name['timeout'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_READINESSCHECK.fields_by_name['check_interval'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_READINESSCHECK.fields_by_name['timeout'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_READINESSCHECK.fields_by_name['app_start_timeout'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_LIVENESSCHECK.fields_by_name['check_interval'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_LIVENESSCHECK.fields_by_name['timeout'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_LIVENESSCHECK.fields_by_name['initial_delay'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
DESCRIPTOR.message_types_by_name['ApiConfigHandler'] = _APICONFIGHANDLER
DESCRIPTOR.message_types_by_name['ErrorHandler'] = _ERRORHANDLER
DESCRIPTOR.message_types_by_name['UrlMap'] = _URLMAP
DESCRIPTOR.message_types_by_name['StaticFilesHandler'] = _STATICFILESHANDLER
DESCRIPTOR.message_types_by_name['ScriptHandler'] = _SCRIPTHANDLER
DESCRIPTOR.message_types_by_name['ApiEndpointHandler'] = _APIENDPOINTHANDLER
DESCRIPTOR.message_types_by_name['HealthCheck'] = _HEALTHCHECK
DESCRIPTOR.message_types_by_name['ReadinessCheck'] = _READINESSCHECK
DESCRIPTOR.message_types_by_name['LivenessCheck'] = _LIVENESSCHECK
DESCRIPTOR.message_types_by_name['Library'] = _LIBRARY
DESCRIPTOR.enum_types_by_name['AuthFailAction'] = _AUTHFAILACTION
DESCRIPTOR.enum_types_by_name['LoginRequirement'] = _LOGINREQUIREMENT
DESCRIPTOR.enum_types_by_name['SecurityLevel'] = _SECURITYLEVEL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ApiConfigHandler = _reflection.GeneratedProtocolMessageType('ApiConfigHandler', (_message.Message,), {
  'DESCRIPTOR' : _APICONFIGHANDLER,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """\ `Google Cloud Endpoints
  <https://cloud.google.com/appengine/docs/python/endpoints/>`__
  configuration for API handlers.
  
  Attributes:
      auth_fail_action:
          Action to take when users access resources that require
          authentication. Defaults to ``redirect``.
      login:
          Level of login required to access this resource. Defaults to
          ``optional``.
      script:
          Path to the script from the application root directory.
      security_level:
          Security (HTTPS) enforcement for this URL.
      url:
          URL to serve the endpoint at.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.ApiConfigHandler)
  })
_sym_db.RegisterMessage(ApiConfigHandler)

ErrorHandler = _reflection.GeneratedProtocolMessageType('ErrorHandler', (_message.Message,), {
  'DESCRIPTOR' : _ERRORHANDLER,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Custom static error page to be served when an error occurs.
  
  Attributes:
      error_code:
          Error condition this handler applies to.
      static_file:
          Static file content to be served for this error.
      mime_type:
          MIME type of file. Defaults to ``text/html``.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.ErrorHandler)
  })
_sym_db.RegisterMessage(ErrorHandler)

UrlMap = _reflection.GeneratedProtocolMessageType('UrlMap', (_message.Message,), {
  'DESCRIPTOR' : _URLMAP,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """URL pattern and description of how the URL should be handled. App
  Engine can handle URLs by executing application code or by serving
  static files uploaded with the version, such as images, CSS, or
  JavaScript.
  
  Attributes:
      url_regex:
          URL prefix. Uses regular expression syntax, which means regexp
          special characters must be escaped, but should not contain
          groupings. All URLs that begin with this prefix are handled by
          this handler, using the portion of the URL after the prefix as
          part of the file path.
      handler_type:
          Type of handler for this URL pattern.
      static_files:
          Returns the contents of a file, such as an image, as the
          response.
      script:
          Executes a script to handle the requests that match this URL
          pattern. Only the ``auto`` value is supported for Node.js in
          the App Engine standard environment, for example ``"script":
          "auto"``.
      api_endpoint:
          Uses API Endpoints to handle requests.
      security_level:
          Security (HTTPS) enforcement for this URL.
      login:
          Level of login required to access this resource. Not supported
          for Node.js in the App Engine standard environment.
      auth_fail_action:
          Action to take when users access resources that require
          authentication. Defaults to ``redirect``.
      redirect_http_response_code:
          \ ``30x`` code to use when performing redirects for the
          ``secure`` field. Defaults to ``302``.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.UrlMap)
  })
_sym_db.RegisterMessage(UrlMap)

StaticFilesHandler = _reflection.GeneratedProtocolMessageType('StaticFilesHandler', (_message.Message,), {

  'HttpHeadersEntry' : _reflection.GeneratedProtocolMessageType('HttpHeadersEntry', (_message.Message,), {
    'DESCRIPTOR' : _STATICFILESHANDLER_HTTPHEADERSENTRY,
    '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
    # @@protoc_insertion_point(class_scope:google.appengine.v1.StaticFilesHandler.HttpHeadersEntry)
    })
  ,
  'DESCRIPTOR' : _STATICFILESHANDLER,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Files served directly to the user for a given URL, such as images, CSS
  stylesheets, or JavaScript source files. Static file handlers describe
  which files in the application directory are static files, and which
  URLs serve them.
  
  Attributes:
      path:
          Path to the static files matched by the URL pattern, from the
          application root directory. The path can refer to text matched
          in groupings in the URL pattern.
      upload_path_regex:
          Regular expression that matches the file paths for all files
          that should be referenced by this handler.
      http_headers:
          HTTP headers to use for all responses from these URLs.
      mime_type:
          MIME type used to serve all files served by this handler.
          Defaults to file-specific MIME types, which are derived from
          each file’s filename extension.
      expiration:
          Time a static file served by this handler should be cached by
          web proxies and browsers.
      require_matching_file:
          Whether this handler should match the request if the file
          referenced by the handler does not exist.
      application_readable:
          Whether files should also be uploaded as code data. By
          default, files declared in static file handlers are uploaded
          as static data and are only served to end users; they cannot
          be read by the application. If enabled, uploads are charged
          against both your code and static data storage resource
          quotas.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.StaticFilesHandler)
  })
_sym_db.RegisterMessage(StaticFilesHandler)
_sym_db.RegisterMessage(StaticFilesHandler.HttpHeadersEntry)

ScriptHandler = _reflection.GeneratedProtocolMessageType('ScriptHandler', (_message.Message,), {
  'DESCRIPTOR' : _SCRIPTHANDLER,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Executes a script to handle the request that matches the URL pattern.
  
  Attributes:
      script_path:
          Path to the script from the application root directory.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.ScriptHandler)
  })
_sym_db.RegisterMessage(ScriptHandler)

ApiEndpointHandler = _reflection.GeneratedProtocolMessageType('ApiEndpointHandler', (_message.Message,), {
  'DESCRIPTOR' : _APIENDPOINTHANDLER,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Uses Google Cloud Endpoints to handle requests.
  
  Attributes:
      script_path:
          Path to the script from the application root directory.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.ApiEndpointHandler)
  })
_sym_db.RegisterMessage(ApiEndpointHandler)

HealthCheck = _reflection.GeneratedProtocolMessageType('HealthCheck', (_message.Message,), {
  'DESCRIPTOR' : _HEALTHCHECK,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Health checking configuration for VM instances. Unhealthy instances
  are killed and replaced with new instances. Only applicable for
  instances in App Engine flexible environment.
  
  Attributes:
      disable_health_check:
          Whether to explicitly disable health checks for this instance.
      host:
          Host header to send when performing an HTTP health check.
          Example: “myapp.appspot.com”
      healthy_threshold:
          Number of consecutive successful health checks required before
          receiving traffic.
      unhealthy_threshold:
          Number of consecutive failed health checks required before
          removing traffic.
      restart_threshold:
          Number of consecutive failed health checks required before an
          instance is restarted.
      check_interval:
          Interval between health checks.
      timeout:
          Time before the health check is considered failed.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.HealthCheck)
  })
_sym_db.RegisterMessage(HealthCheck)

ReadinessCheck = _reflection.GeneratedProtocolMessageType('ReadinessCheck', (_message.Message,), {
  'DESCRIPTOR' : _READINESSCHECK,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Readiness checking configuration for VM instances. Unhealthy instances
  are removed from traffic rotation.
  
  Attributes:
      path:
          The request path.
      host:
          Host header to send when performing a HTTP Readiness check.
          Example: “myapp.appspot.com”
      failure_threshold:
          Number of consecutive failed checks required before removing
          traffic.
      success_threshold:
          Number of consecutive successful checks required before
          receiving traffic.
      check_interval:
          Interval between health checks.
      timeout:
          Time before the check is considered failed.
      app_start_timeout:
          A maximum time limit on application initialization, measured
          from moment the application successfully replies to a
          healthcheck until it is ready to serve traffic.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.ReadinessCheck)
  })
_sym_db.RegisterMessage(ReadinessCheck)

LivenessCheck = _reflection.GeneratedProtocolMessageType('LivenessCheck', (_message.Message,), {
  'DESCRIPTOR' : _LIVENESSCHECK,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Health checking configuration for VM instances. Unhealthy instances
  are killed and replaced with new instances.
  
  Attributes:
      path:
          The request path.
      host:
          Host header to send when performing a HTTP Liveness check.
          Example: “myapp.appspot.com”
      failure_threshold:
          Number of consecutive failed checks required before
          considering the VM unhealthy.
      success_threshold:
          Number of consecutive successful checks required before
          considering the VM healthy.
      check_interval:
          Interval between health checks.
      timeout:
          Time before the check is considered failed.
      initial_delay:
          The initial delay before starting to execute the checks.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.LivenessCheck)
  })
_sym_db.RegisterMessage(LivenessCheck)

Library = _reflection.GeneratedProtocolMessageType('Library', (_message.Message,), {
  'DESCRIPTOR' : _LIBRARY,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Third-party Python runtime library that is required by the
  application.
  
  Attributes:
      name:
          Name of the library. Example: “django”.
      version:
          Version of the library to select, or “latest”.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.Library)
  })
_sym_db.RegisterMessage(Library)


DESCRIPTOR._options = None
_SECURITYLEVEL._options = None
_ERRORHANDLER_ERRORCODE._options = None
_STATICFILESHANDLER_HTTPHEADERSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Wrapper for JSON-based Component Status."""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import six


class ComponentStatus(object):
  """Class that wraps a KubeRun Component Status JSON object."""

  def __init__(self,
               name,
               deployment_state,
               commit_id,
               deployment_time,
               url,
               log_url,
               services=None,
               deployment_message='',
               deployment_reason=''):
    self.name = name
    self.deployment_state = deployment_state
    self.commit_id = commit_id
    self.deployment_time = deployment_time
    self.url = url
    self.log_url = log_url
    self.services = [] if services is None else services
    self.deployment_message = deployment_message
    self.deployment_reason = deployment_reason

  @classmethod
  def FromJSON(cls, name, json_object):
    return cls(
        name=name,
        deployment_state=json_object['deploymentState'],
        deployment_message=json_object['deploymentMessage'],
        commit_id=json_object['commitId'],
        deployment_time=json_object['deploymentTimestamp'],
        url=json_object['url'],
        log_url=json_object['logUrl'],
        services=[
            r['name']
            for r in json_object['resources']
            if r['type'] == 'Service'
        ],
        deployment_reason=json_object['deploymentReason']
        if 'deploymentReason' in json_object else '')

  def __repr__(self):
    # TODO(b/171419038): Create a common base class for these data wrappers
    items = sorted(self.__dict__.items())
    attrs_as_kv_strings = ['{}={!r}'.format(k, v) for (k, v) in items]
    return six.text_type('ComponentStatus({})').format(
        ', '.join(attrs_as_kv_strings))

  def __eq__(self, other):
    if isinstance(other, self.__class__):
      return self.__dict__ == other.__dict__
    return False



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""composite-types command basics."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.deployment_manager import exceptions
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.command_lib.deployment_manager import importer
from googlecloudsdk.core import properties


def AddCompositeTypeNameFlag(parser):
  """Add the composite type name argument.

  Args:
    parser: An argparse parser that you can use to add arguments that go
        on the command line after this command. Positional arguments are
        allowed.
  """
  parser.add_argument('name', help='Composite type name.')


def AddDescriptionFlag(parser):
  """Add the description argument.

  Args:
    parser: An argparse parser that you can use to add arguments that go
        on the command line after this command. Positional arguments are
        allowed.
  """
  parser.add_argument('--description',
                      help='Optional description of the composite type.',
                      default='')


def AddStatusFlag(parser):
  """Add the status argument.

  Args:
    parser: An argparse parser that you can use to add arguments that go
        on the command line after this command. Positional arguments are
        allowed.
  """
  parser.add_argument('--status',
                      help='Optional status for a composite type.',
                      choices=['DEPRECATED', 'EXPERIMENTAL', 'SUPPORTED'],
                      default=None)


template_flag_arg_type = arg_parsers.RegexpValidator(
    r'.*\.(py|jinja)',
    'must be a python (".py") or jinja (".jinja") file')


def AddTemplateFlag(parser):
  """Add the template path argument.

  Args:
    parser: An argparse parser that you can use to add arguments that go
        on the command line after this command. Positional arguments are
        allowed.
  """
  parser.add_argument('--template',
                      help=('Path to a python or jinja file (local or via URL) '
                            'that defines the composite type. If you want to '
                            'provide a schema, that file must be in the same '
                            'location: e.g. "--template=./foobar.jinja" means '
                            '"./foobar.jinja.schema" should also exist. The '
                            'file must end in either ".jinja" or ".py" to be '
                            'interpreted correctly.'),
                      type=template_flag_arg_type,
                      required=True)


def TemplateContentsFor(messages, template_path):
  """Build a TemplateContents message from a local template or url.

  Args:
    messages: The API message to use.
    template_path: Path to the config yaml file, with an optional list of
      imports.

  Returns:
    The TemplateContents message from the template at template_path.

  Raises:
    Error if the provided file is not a template.
  """
  config_obj = importer.BuildConfig(template=template_path)

  if not config_obj.IsTemplate():
    raise exceptions.Error(
        'The provided file must be a template.')

  template_name = config_obj.GetBaseName()
  schema_name = template_name + '.schema'
  file_type = messages.TemplateContents.InterpreterValueValuesEnum.JINJA if template_name.endswith(
      '.jinja') else messages.TemplateContents.InterpreterValueValuesEnum.PYTHON

  imports = importer.CreateImports(messages, config_obj)

  template = ''
  schema = ''

  # Find schema and template from imports
  for item in imports:
    if item.name == template_name:
      template = item.content
    elif item.name == schema_name:
      schema = item.content

  # Remove schema and template from imports
  imports = [item for item in imports
             if item.name not in [template_name, schema_name]]

  return messages.TemplateContents(imports=imports,
                                   schema=schema,
                                   template=template,
                                   interpreter=file_type)


def GetReference(resources, name):
  return resources.Parse(
      name,
      params={'project': properties.VALUES.core.project.GetOrFail},
      collection='deploymentmanager.compositeTypes')



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module used by gcloud to communicate with appengine services."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from __future__ import with_statement

from googlecloudsdk.api_lib.app import util
from googlecloudsdk.api_lib.app import yaml_parsing
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.credentials import creds as c_creds
from googlecloudsdk.core.credentials import http
from googlecloudsdk.core.credentials import store as c_store
from googlecloudsdk.third_party.appengine.datastore import datastore_index
from googlecloudsdk.third_party.appengine.tools import appengine_rpc_httplib2
from oauth2client.contrib import gce as oauth2client_gce
import six
import six.moves.urllib.error
import six.moves.urllib.parse
import six.moves.urllib.request


APPCFG_SCOPES = ['https://www.googleapis.com/auth/cloud-platform']

# Parameters for reading from the GCE metadata service.
METADATA_BASE = 'http://metadata.google.internal'
SERVICE_ACCOUNT_BASE = ('computeMetadata/v1/instance/service-accounts/default')

RpcServerClass = appengine_rpc_httplib2.HttpRpcServerOAuth2  # pylint: disable=invalid-name


class Error(exceptions.Error):
  """Base exception for the module."""
  pass


class UnknownConfigType(Error):
  """An exception for when trying to update a config type we don't know."""
  pass


class AppengineClient(object):
  """Client used by gcloud to communicate with appengine services.

  Attributes:
    server: The appengine server to which requests are sent.
    project: The appengine application in use.
    oauth2_access_token: An existing OAuth2 access token to use.
    oauth2_refresh_token: An existing OAuth2 refresh token to use.
    oauth_scopes: The required scope of credentials to call app engine.
    authenticate_service_account: Authenticate using the default service account
      for the Google Compute Engine VM in which gcloud is being called.
    ignore_bad_certs: Whether to ignore certificate errors when talking to the
      server.
    client_id: Client ID of the credentials to use.
    client_secret: Client secret of the credentials to use.
  """

  _PREPARE_TIMEOUT_RETIRES = 15

  def __init__(self, server=None, ignore_bad_certs=False):
    self.server = server or 'appengine.google.com'
    self.project = properties.VALUES.core.project.Get(required=True)
    self.ignore_bad_certs = ignore_bad_certs
    # Auth related options
    self.oauth2_access_token = None
    self.oauth2_refresh_token = None
    self.oauth_scopes = APPCFG_SCOPES
    self.authenticate_service_account = False
    self.client_id = None
    self.client_secret = None

    credentials = c_store.LoadFreshCredential(use_google_auth=True)
    if credentials:
      self._InitCredentials(credentials)

  def _InitCredentials(self, credentials):
    if c_creds.IsGoogleAuthCredentials(credentials):
      self._InitGoogleAuthCreds(credentials)
    else:
      self._InitOauth2clientCreds(credentials)

  def _InitGoogleAuthCreds(self, credentials):
    """Initializes the object with a given google-auth credentials."""
    creds_type = c_creds.CredentialTypeGoogleAuth.FromCredentials(credentials)
    if creds_type in (c_creds.CredentialTypeGoogleAuth.SERVICE_ACCOUNT,
                      c_creds.CredentialTypeGoogleAuth.P12_SERVICE_ACCOUNT,
                      c_creds.CredentialTypeGoogleAuth.DEVSHELL):
      self.oauth2_access_token = credentials.token
    elif creds_type == c_creds.CredentialTypeGoogleAuth.GCE:
      self.authenticate_service_account = True
    elif creds_type == c_creds.CredentialTypeGoogleAuth.USER_ACCOUNT:
      self.oauth2_access_token = credentials.token
      self.oauth2_refresh_token = credentials.refresh_token
      self.client_id = credentials.client_id
      self.client_secret = credentials.client_secret

  def _InitOauth2clientCreds(self, credentials):
    """Initializes the object with a given oauth2client credentials."""
    creds_type = c_creds.CredentialType.FromCredentials(credentials)

    if creds_type in (c_creds.CredentialType.SERVICE_ACCOUNT,
                      c_creds.CredentialType.P12_SERVICE_ACCOUNT,
                      c_creds.CredentialType.DEVSHELL):
      self.oauth2_access_token = credentials.access_token
    elif creds_type == c_creds.CredentialType.GCE:
      self.authenticate_service_account = True
    elif creds_type == c_creds.CredentialType.USER_ACCOUNT:
      self.oauth2_access_token = credentials.access_token
      self.oauth2_refresh_token = credentials.refresh_token
      self.client_id = credentials.client_id
      self.client_secret = credentials.client_secret

  def CleanupIndexes(self, index_yaml):
    """Removes unused datastore indexes.

    Args:
      index_yaml: The parsed yaml file with index data.
    """
    rpcserver = self._GetRpcServer()
    response = rpcserver.Send('/api/datastore/index/diff',
                              app_id=self.project, payload=index_yaml.ToYAML())
    unused_new_indexes, notused_indexes = (
        datastore_index.ParseMultipleIndexDefinitions(response))

    # Get confirmation from user which indexes should be deleted.
    deletions = datastore_index.IndexDefinitions(indexes=[])
    if notused_indexes.indexes:
      for index in notused_indexes.indexes:
        msg = ('This index is no longer defined in your index.yaml file.\n{0}'
               .format(six.text_type((index.ToYAML()))))
        prompt = 'Do you want to delete this index'
        if console_io.PromptContinue(msg, prompt, default=True):
          deletions.indexes.append(index)

    # Do deletions of confirmed indexes.
    if deletions.indexes:
      response = rpcserver.Send('/api/datastore/index/delete',
                                app_id=self.project, payload=deletions.ToYAML())
      not_deleted = datastore_index.ParseIndexDefinitions(response)

      # Notify the user when indexes are not deleted.
      if not_deleted.indexes:
        not_deleted_count = len(not_deleted.indexes)
        if not_deleted_count == 1:
          warning_message = ('An index was not deleted.  Most likely this is '
                             'because it no longer exists.\n\n')
        else:
          warning_message = ('%d indexes were not deleted.  Most likely this '
                             'is because they no longer exist.\n\n'
                             % not_deleted_count)
        for index in not_deleted.indexes:
          warning_message += index.ToYAML()
        log.warning(warning_message)

  def PrepareVmRuntime(self):
    """Prepare the application for vm runtimes and return state."""
    rpcserver = self._GetRpcServer(
        timeout_max_errors=self._PREPARE_TIMEOUT_RETIRES)
    rpcserver.Send('/api/vms/prepare', app_id=self.project)

  def UpdateConfig(self, config_name, parsed_yaml):
    """Updates any of the supported config file types.

    Args:
      config_name: str, The name of the config to deploy.
      parsed_yaml: The parsed object corresponding to that config type.

    Raises:
      UnknownConfigType: If config_name is not a value config type.

    Returns:
      Whatever the underlying update methods return.
    """
    if config_name == yaml_parsing.ConfigYamlInfo.CRON:
      return self.UpdateCron(parsed_yaml)
    if config_name == yaml_parsing.ConfigYamlInfo.DISPATCH:
      return self.UpdateDispatch(parsed_yaml)
    if config_name == yaml_parsing.ConfigYamlInfo.DOS:
      return self.UpdateDos(parsed_yaml)
    if config_name == yaml_parsing.ConfigYamlInfo.QUEUE:
      return self.UpdateQueues(parsed_yaml)
    raise UnknownConfigType(
        'Config type [{0}] is not a known config type'.format(config_name))

  def UpdateCron(self, cron_yaml):
    """Updates any new or changed cron definitions.

    Args:
      cron_yaml: The parsed yaml file with cron data.
    """
    self._GetRpcServer().Send('/api/cron/update',
                              app_id=self.project, payload=cron_yaml.ToYAML())

  def UpdateDispatch(self, dispatch_yaml):
    """Updates new or changed dispatch definitions.

    Args:
      dispatch_yaml: The parsed yaml file with dispatch data.
    """
    self._GetRpcServer().Send('/api/dispatch/update',
                              app_id=self.project,
                              payload=dispatch_yaml.ToYAML())

  def UpdateDos(self, dos_yaml):
    """Updates any new or changed dos definitions.

    Args:
      dos_yaml: The parsed yaml file with dos data.
    """
    log.warning(
        '`gcloud app deploy dos.yaml` is deprecated and will be removed '
        'in an upcoming release. Please use `gcloud app firewall-rules` '
        'instead.')
    self._GetRpcServer().Send('/api/dos/update',
                              app_id=self.project, payload=dos_yaml.ToYAML())

  def UpdateQueues(self, queue_yaml):
    """Updates any new or changed task queue definitions.

    Args:
      queue_yaml: The parsed yaml file with queue data.
    """
    self._GetRpcServer().Send('/api/queue/update',
                              app_id=self.project, payload=queue_yaml.ToYAML())

  def _GetRpcServer(self, timeout_max_errors=2):
    """Returns an instance of an AbstractRpcServer.

    Args:
      timeout_max_errors: How many timeout errors to retry.
    Returns:
      A new AbstractRpcServer, on which RPC calls can be made.
    """
    log.debug('Host: {0}'.format(self.server))

    if self._IsGceEnvironment():
      credentials = oauth2client_gce.AppAssertionCredentials()
    else:
      credentials = None

    # In this case, the get_user_credentials parameters to the RPC server
    # constructor is actually an OAuth2Parameters.

    get_user_credentials = (
        appengine_rpc_httplib2.HttpRpcServerOAuth2.OAuth2Parameters(
            access_token=self.oauth2_access_token,
            client_id=self.client_id,
            client_secret=self.client_secret,
            scope=APPCFG_SCOPES,
            refresh_token=self.oauth2_refresh_token,
            credential_file=None,
            token_uri=None,
            credentials=credentials))
    # Also set gflags flag... this is a bit of a hack.
    if hasattr(appengine_rpc_httplib2.tools, 'FLAGS'):
      appengine_rpc_httplib2.tools.FLAGS.auth_local_webserver = True

    server = RpcServerClass(
        self.server,
        get_user_credentials,
        util.GetUserAgent(),
        util.GetSourceName(),
        host_override=None,
        save_cookies=True,
        auth_tries=3,
        timeout_max_errors=timeout_max_errors,
        account_type='HOSTED_OR_GOOGLE',
        secure=True,
        ignore_certs=self.ignore_bad_certs,
        http_object=http.Http())
    # TODO(b/36050949) Hack to avoid failure due to missing cacerts.txt
    # resource.
    server.certpath = None
    # Don't use a cert file if the user passed ignore-bad-certs.
    server.cert_file_available = not self.ignore_bad_certs
    return util.RPCServer(server)

  def _IsGceEnvironment(self):
    """Determine if we are running in a GCE environment.

    Returns:
      True if we are running in a GCE environment.

    Raises:
      Error: The user has requested authentication for a service account but the
      environment is not correct for that to work.
    """
    if self.authenticate_service_account:
      # Avoid hard-to-understand errors later by checking that we have a
      # metadata service (so we are in a GCE VM) and that the VM is configured
      # with access to the appengine.admin scope.
      url = '%s/%s/scopes' % (METADATA_BASE, SERVICE_ACCOUNT_BASE)
      try:
        req = six.moves.urllib.request.Request(
            url, headers={'Metadata-Flavor': 'Google'})
        vm_scopes_string = six.moves.urllib.request.urlopen(req).read()
        vm_scopes_string = six.ensure_text(vm_scopes_string)
      except six.moves.urllib.error.URLError as e:
        raise Error(
            'Could not obtain scope list from metadata service: %s: %s. This '
            'may be because we are not running in a Google Compute Engine VM.' %
            (url, e))
      vm_scopes = vm_scopes_string.split()
      missing = list(set(self.oauth_scopes).difference(vm_scopes))
      if missing:
        raise Error(
            'You are currently logged into gcloud using a service account '
            'which does not have the appropriate access to [{0}]. The account '
            'has the following scopes: [{1}]. It needs [{2}] in order to '
            'succeed.\nPlease recreate this VM instance with the missing '
            'scopes. You may also log into a standard account that has the '
            'appropriate access by using `gcloud auth login`.'
            .format(self.project, ', '.join(vm_scopes), ', '.join(missing)))
      return True
    else:
      return False



# Copyright 2016 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import os
import sys

import mock
import pytest  # type: ignore
from six.moves import reload_module

try:
    import oauth2client.client  # type: ignore
    import oauth2client.contrib.gce  # type: ignore
    import oauth2client.service_account  # type: ignore
except ImportError:  # pragma: NO COVER
    pytest.skip(
        "Skipping oauth2client tests since oauth2client is not installed.",
        allow_module_level=True,
    )

from google.auth import _oauth2client


DATA_DIR = os.path.join(os.path.dirname(__file__), "data")
SERVICE_ACCOUNT_JSON_FILE = os.path.join(DATA_DIR, "service_account.json")


def test__convert_oauth2_credentials():
    old_credentials = oauth2client.client.OAuth2Credentials(
        "access_token",
        "client_id",
        "client_secret",
        "refresh_token",
        datetime.datetime.min,
        "token_uri",
        "user_agent",
        scopes="one two",
    )

    new_credentials = _oauth2client._convert_oauth2_credentials(old_credentials)

    assert new_credentials.token == old_credentials.access_token
    assert new_credentials._refresh_token == old_credentials.refresh_token
    assert new_credentials._client_id == old_credentials.client_id
    assert new_credentials._client_secret == old_credentials.client_secret
    assert new_credentials._token_uri == old_credentials.token_uri
    assert new_credentials.scopes == old_credentials.scopes


def test__convert_service_account_credentials():
    old_class = oauth2client.service_account.ServiceAccountCredentials
    old_credentials = old_class.from_json_keyfile_name(SERVICE_ACCOUNT_JSON_FILE)

    new_credentials = _oauth2client._convert_service_account_credentials(
        old_credentials
    )

    assert (
        new_credentials.service_account_email == old_credentials.service_account_email
    )
    assert new_credentials._signer.key_id == old_credentials._private_key_id
    assert new_credentials._token_uri == old_credentials.token_uri


def test__convert_service_account_credentials_with_jwt():
    old_class = oauth2client.service_account._JWTAccessCredentials
    old_credentials = old_class.from_json_keyfile_name(SERVICE_ACCOUNT_JSON_FILE)

    new_credentials = _oauth2client._convert_service_account_credentials(
        old_credentials
    )

    assert (
        new_credentials.service_account_email == old_credentials.service_account_email
    )
    assert new_credentials._signer.key_id == old_credentials._private_key_id
    assert new_credentials._token_uri == old_credentials.token_uri


def test__convert_gce_app_assertion_credentials():
    old_credentials = oauth2client.contrib.gce.AppAssertionCredentials(
        email="some_email"
    )

    new_credentials = _oauth2client._convert_gce_app_assertion_credentials(
        old_credentials
    )

    assert (
        new_credentials.service_account_email == old_credentials.service_account_email
    )


@pytest.fixture
def mock_oauth2client_gae_imports(mock_non_existent_module):
    mock_non_existent_module("google.appengine.api.app_identity")
    mock_non_existent_module("google.appengine.ext.ndb")
    mock_non_existent_module("google.appengine.ext.webapp.util")
    mock_non_existent_module("webapp2")


@mock.patch("google.auth.app_engine.app_identity")
def test__convert_appengine_app_assertion_credentials(
    app_identity, mock_oauth2client_gae_imports
):

    import oauth2client.contrib.appengine  # type: ignore

    service_account_id = "service_account_id"
    old_credentials = oauth2client.contrib.appengine.AppAssertionCredentials(
        scope="one two", service_account_id=service_account_id
    )

    new_credentials = _oauth2client._convert_appengine_app_assertion_credentials(
        old_credentials
    )

    assert new_credentials.scopes == ["one", "two"]
    assert new_credentials._service_account_id == old_credentials.service_account_id


class FakeCredentials(object):
    pass


def test_convert_success():
    convert_function = mock.Mock(spec=["__call__"])
    conversion_map_patch = mock.patch.object(
        _oauth2client, "_CLASS_CONVERSION_MAP", {FakeCredentials: convert_function}
    )
    credentials = FakeCredentials()

    with conversion_map_patch:
        result = _oauth2client.convert(credentials)

    convert_function.assert_called_once_with(credentials)
    assert result == convert_function.return_value


def test_convert_not_found():
    with pytest.raises(ValueError) as excinfo:
        _oauth2client.convert("a string is not a real credentials class")

    assert excinfo.match("Unable to convert")


@pytest.fixture
def reset__oauth2client_module():
    """Reloads the _oauth2client module after a test."""
    reload_module(_oauth2client)


def test_import_has_app_engine(
    mock_oauth2client_gae_imports, reset__oauth2client_module
):
    reload_module(_oauth2client)
    assert _oauth2client._HAS_APPENGINE


def test_import_without_oauth2client(monkeypatch, reset__oauth2client_module):
    monkeypatch.setitem(sys.modules, "oauth2client", None)
    with pytest.raises(ImportError) as excinfo:
        reload_module(_oauth2client)

    assert excinfo.match("oauth2client")



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""JSON-based domain mapping wrapper."""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.kuberun import kubernetesobject
from googlecloudsdk.api_lib.kuberun import structuredout


class DomainMapping(kubernetesobject.KubernetesObject):
  """Wraps JSON-based dict object of a domain mapping."""

  @property
  def routeName(self):
    return self._props['spec']['routeName']

  @property
  def records(self):
    return [
        ResourceRecord(r)
        for r in self._props['status'].get('resourceRecords', [])
    ]


class ResourceRecord(structuredout.MapObject):
  """Wraps JSON-based dict object of a resource record of a domain mapping."""

  @property
  def type(self):
    return self._props['type']

  @property
  def rrdata(self):
    return self._props['rrdata']

  @property
  def name(self):
    return self._props.get('name')

  @name.setter
  def name(self, n):
    self._props['name'] = n



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Common helper methods for Genomics commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import abc
import re
from googlecloudsdk.api_lib.genomics import genomics_util
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.core import resources
import six


def CreateFromName(name):
  """Creates a client and resource reference for a name.

  Args:
    name: An operation name, optionally including projects/, operations/, and a
        project name.

  Returns:
    A tuple containing the genomics client and resource reference.
  """
  client = None

  if re.search('[a-zA-Z]', name.split('/')[-1]):
    client = GenomicsV1ApiClient()
  else:
    client = GenomicsV2ApiClient()

  return client, client.ResourceFromName(name)


class GenomicsApiClient(six.with_metaclass(abc.ABCMeta, object)):
  """Base class for clients for accessing the genomics API.
  """

  def __init__(self, version):
    self._messages = genomics_util.GetGenomicsMessages(version)
    self._client = genomics_util.GetGenomicsClient(version)
    self._registry = resources.REGISTRY.Clone()
    self._registry.RegisterApiByName('genomics', version)

  @abc.abstractmethod
  def ResourceFromName(self, name):
    raise NotImplementedError()

  @abc.abstractmethod
  def Poller(self):
    raise NotImplementedError()

  @abc.abstractmethod
  def GetOperation(self, resource):
    raise NotImplementedError()

  @abc.abstractmethod
  def CancelOperation(self, resource):
    raise NotImplementedError()


class GenomicsV1ApiClient(GenomicsApiClient):
  """Client for accessing the V1 genomics API.
  """

  def __init__(self):
    super(GenomicsV1ApiClient, self).__init__('v1')

  def ResourceFromName(self, name):
    return self._registry.Parse(name, collection='genomics.operations')

  def Poller(self):
    return waiter.CloudOperationPollerNoResources(self._client.operations)

  def GetOperation(self, resource):
    return self._client.operations.Get(
        self._messages.GenomicsOperationsGetRequest(
            name=resource.RelativeName()))

  def CancelOperation(self, resource):
    return self._client.operations.Cancel(
        self._messages.GenomicsOperationsCancelRequest(
            name=resource.RelativeName()))


class GenomicsV2ApiClient(GenomicsApiClient):
  """Client for accessing the V2 genomics API.
  """

  def __init__(self):
    super(GenomicsV2ApiClient, self).__init__('v2alpha1')

  def ResourceFromName(self, name):
    return self._registry.Parse(
        name, collection='genomics.projects.operations',
        params={'projectsId': genomics_util.GetProjectId()})

  def Poller(self):
    return waiter.CloudOperationPollerNoResources(
        self._client.projects_operations)

  def GetOperation(self, resource):
    return self._client.projects_operations.Get(
        self._messages.GenomicsProjectsOperationsGetRequest(
            name=resource.RelativeName()))

  def CancelOperation(self, resource):
    return self._client.projects_operations.Cancel(
        self._messages.GenomicsProjectsOperationsCancelRequest(
            name=resource.RelativeName()))



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Deletes an AlloyDB instance."""


from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


from googlecloudsdk.api_lib.alloydb import api_util
from googlecloudsdk.api_lib.alloydb import instance_operations
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.alloydb import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources
from googlecloudsdk.core.console import console_io


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Delete(base.DeleteCommand):
  """Deletes an AlloyDB instance within a given cluster."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
        To delete an instance, run:

          $ {command} my-instance --cluster=my-cluster --region=us-central1
        """,
  }

  @staticmethod
  def Args(parser):
    """Specifies additional command flags.

    Args:
      parser: argparse.Parser, Parser object for command line inputs
    """
    base.ASYNC_FLAG.AddToParser(parser)
    flags.AddCluster(parser, False)
    flags.AddInstance(parser)
    flags.AddRegion(parser)

  def Run(self, args):
    """Constructs and sends request.

    Args:
      args: argparse.Namespace, An object that contains the values for the
          arguments specified in the .Args() method.

    Returns:
      ProcessHttpResponse of the request made.
    """
    client = api_util.AlloyDBClient(self.ReleaseTrack())
    alloydb_client = client.alloydb_client
    alloydb_messages = client.alloydb_messages
    instance_ref = client.resource_parser.Create(
        'alloydb.projects.locations.clusters.instances',
        projectsId=properties.VALUES.core.project.GetOrFail,
        locationsId=args.region,
        clustersId=args.cluster,
        instancesId=args.instance)

    prompt_message = (
        'Instance settings and IPs will be deleted on upon deletion.')
    if not console_io.PromptContinue(message=prompt_message):
      return None

    req = alloydb_messages.AlloydbProjectsLocationsClustersInstancesDeleteRequest(
        name=instance_ref.RelativeName())
    op = alloydb_client.projects_locations_clusters_instances.Delete(req)
    op_ref = resources.REGISTRY.ParseRelativeName(
        op.name, collection='alloydb.projects.locations.operations')
    log.status.Print('Operation ID: {}'.format(op_ref.Name()))
    if not args.async_:
      instance_operations.Await(op_ref, 'Deleting instance', self.ReleaseTrack(), False)
    return op



# -*- coding: utf-8 -*- #
# Copyright 2020 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""API interface for interacting with cloud storage providers."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import enum

from googlecloudsdk.command_lib.storage import storage_url


class Capability(enum.Enum):
  """Used to track API capabilities relevant to logic in tasks."""
  COMPOSE_OBJECTS = 'COMPOSE_OBJECTS'
  CLIENT_SIDE_HASH_VALIDATION = 'CLIENT_SIDE_HASH_VALIDATION'
  ENCRYPTION = 'ENCRYPTION'
  RESUMABLE_UPLOAD = 'RESUMABLE_UPLOAD'
  SLICED_DOWNLOAD = 'SLICED_DOWNLOAD'
  # For daisy chain operations, the upload stream is not purely seekable.
  # For certain seek calls, we raise errors to avoid re-downloading the object.
  # We do not want the "seekable" method for the upload stream to always return
  # False because in case of GCS, Apitools checks for this value to determine
  # if a resumable upload can be performed. However, for S3,
  # boto3's upload_fileobj calls "seek" with
  # unsupported offset and whence combinations, and to avoid that,
  # we need to mark the upload stream as non-seekable for S3.
  # This value is used by daisy chain operation to determine if the upload
  # stream can be treated as seekable.
  DAISY_CHAIN_SEEKABLE_UPLOAD_STREAM = 'DAISY_CHAIN_SEEKABLE_UPLOAD_STREAM'


class DownloadStrategy(enum.Enum):
  """Enum class for specifying download strategy."""
  ONE_SHOT = 'oneshot'  # No in-flight retries performed.
  # Operations are retried on network errors.
  RETRIABLE_IN_FLIGHT = 'retriable_in_flight'
  # In addition to retrying on errors, operations can be resumed if halted.
  # This option will write tracker files to track the downloads in progress.
  RESUMABLE = 'resumable'


class UploadStrategy(enum.Enum):
  """Enum class for specifying upload strategy."""
  SIMPLE = 'simple'
  RESUMABLE = 'resumable'
  STREAMING = 'streaming'


class NotificationEventType(enum.Enum):
  """Used to filter what events a notification configuration notifies on."""
  OBJECT_ARCHIVE = 'OBJECT_ARCHIVE'
  OBJECT_DELETE = 'OBJECT_DELETE'
  OBJECT_FINALIZE = 'OBJECT_FINALIZE'
  OBJECT_METADATA_UPDATE = 'OBJECT_METADATA_UPDATE'


class NotificationPayloadFormat(enum.Enum):
  """Used to format the body of notifications."""
  JSON = 'json'
  NONE = 'none'


class FieldsScope(enum.Enum):
  """Values used to determine fields and projection values for API calls."""
  FULL = 1
  NO_ACL = 2
  SHORT = 3


class HmacKeyState(enum.Enum):
  ACTIVE = 'ACTIVE'
  INACTIVE = 'INACTIVE'


DEFAULT_PROVIDER = storage_url.ProviderPrefix.GCS
NUM_ITEMS_PER_LIST_PAGE = 1000


class CloudApi(object):
  """Abstract base class for interacting with cloud storage providers.

  Implementations of the Cloud API are not guaranteed to be thread-safe.
  Behavior when calling a Cloud API instance simultaneously across
  threads is undefined and doing so will likely cause errors. Therefore,
  a separate instance of the Cloud API should be instantiated per-thread.

  Attributes:
    capabilities (set[Capability]): If a Capability is present in this set, this
      API can be used to execute related logic in tasks.
  """
  capabilities = set()

  # Some APIs limit the number of objects that can be composed in a single call.
  # This field should be overidden by those APIs, and default to 1 for APIs
  # that do not support compose_objects.
  MAX_OBJECTS_PER_COMPOSE_CALL = 1

  def create_bucket(self, bucket_resource, request_config, fields_scope=None):
    """Creates a new bucket with the specified metadata.

    Args:
      bucket_resource (resource_reference.UnknownResource): The bucket to
        create.
      request_config (RequestConfig): Contains metadata for new bucket.
      fields_scope (FieldsScope): Determines the fields and projection
        parameters of API call.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
      ValueError: Invalid fields_scope.

    Returns:
      resource_reference.BucketResource representing new bucket.
    """
    raise NotImplementedError('create_bucket must be overridden.')

  def delete_bucket(self, bucket_name, request_config):
    """Deletes a bucket.

    Args:
      bucket_name (str): Name of the bucket to delete.
      request_config (RequestConfig): Object containing general API function
        arguments. Subclasses for specific cloud providers are available.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('delete_bucket must be overridden.')

  def get_bucket(self, bucket_name, fields_scope=None):
    """Gets bucket metadata.

    Args:
      bucket_name (str): Name of the bucket.
      fields_scope (FieldsScope): Determines the fields and projection
        parameters of API call.

    Returns:
      resource_reference.BucketResource containing the bucket metadata.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
      ValueError: Invalid fields_scope.
    """
    raise NotImplementedError('get_bucket must be overridden.')

  def get_bucket_iam_policy(self, bucket_name):
    """Gets bucket IAM policy.

    Args:
      bucket_name (str): Name of the bucket.

    Returns:
      Provider-specific data type. Currently, only available for GCS so returns
        Apitools messages.Policy object. If supported for
        more providers in the future, use a generic container.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('get_bucket_iam_policy must be overridden.')

  def list_buckets(self, fields_scope=None):
    """Lists bucket metadata for the given project.

    Args:
      fields_scope (FieldsScope): Determines the fields and projection
        parameters of API call.

    Yields:
      Iterator over resource_reference.BucketResource objects

    Raises:
      NotImplementedError: This function was not implemented by a class using
        this interface.
      ValueError: Invalid fields_scope.
    """
    raise NotImplementedError('list_buckets must be overridden.')

  def lock_bucket_retention_policy(self, bucket_resource, request_config):
    """Locks a bucket's retention policy.

    Args:
      bucket_resource (UnknownResource): The bucket with the policy to lock.
      request_config (RequestConfig): Object containing general API function
        arguments. Subclasses for specific cloud providers are available.

    Returns:
      resource_reference.BucketResource containing the bucket metadata.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
          this interface.
    """
    raise NotImplementedError(
        'lock_bucket_retention_policy must be overridden.')

  def patch_bucket(self, bucket_resource, request_config, fields_scope=None):
    """Patches bucket metadata.

    Args:
      bucket_resource (BucketResource|UnknownResource): The bucket to patch.
      request_config (RequestConfig): Contains new metadata for the bucket.
      fields_scope (FieldsScope): Determines the fields and projection
        parameters of API call.

    Returns:
      resource_reference.BucketResource containing the bucket metadata.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
          this interface.
      ValueError: Invalid fields_scope.
    """
    raise NotImplementedError('patch_bucket must be overridden.')

  def set_bucket_iam_policy(self, bucket_name, policy):
    """Sets bucket IAM policy.

    Args:
      bucket_name (str): Name of the bucket.
      policy (object): Provider-specific data type. Currently, only
        available for GCS so Apitools messages.Policy object. If supported for
        more providers in the future, use a generic container.

    Returns:
      Provider-specific data type. Currently, only available for GCS so returns
        Apitools messages.Policy object. If supported for
        more providers in the future, use a generic container.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('get_bucket_iam_policy must be overridden.')

  def create_hmac_key(self, service_account_email):
    """Creates an HMAC key.

    Args:
      service_account_email (str): The email of the service account to use.

    Returns:
      gcs_resource_reference.GcsHmacKeyResource. Provider-specific data type
      is used for now because we currently support this feature only for the
      JSON API.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('create_hmac_key must be overridden.')

  def delete_hmac_key(self, access_id):
    """Deletes an HMAC key.

    Args:
      access_id (str): The access ID corresponding to the HMAC key.

    Returns:
      None

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('delete_hmac_key must be overridden.')

  def get_hmac_key(self, access_id):
    """Gets an HMAC key.

    Args:
      access_id (str): The access ID corresponding to the HMAC key.

    Returns:
      gcs_resource_reference.GcsHmacKeyResource. Provider-specific data type
      is used for now because we currently support this feature only for the
      JSON API.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('get_hmac_key must be overridden.')

  def list_hmac_keys(self, service_account_email=None, show_deleted_keys=False,
                     fields_scope=None):
    """Lists HMAC keys.

    Args:
      service_account_email (str): Return HMAC keys for the given service
        account email.
      show_deleted_keys (bool): If True, include keys in the DELETED state.
      fields_scope (FieldsScope): Determines which metadata keys
        the API should return for each key.

    Yields:
      Iterator over gcs_resource_reference.GcsHmacKeyResource objects.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    pass

  def patch_hmac_key(self, access_id, etag, state):
    """Updates an HMAC key.

    Args:
      access_id (str): The access ID corresponding to the HMAC key.
      etag (str): Only perform the patch request if the etag matches this value.
      state (HmacKeyState): The desired state of the HMAC key.

    Returns:
      gcs_resource_reference.GcsHmacKeyResource. Provider-specific data type
      is used for now because we currently support this feature only for the
      JSON API.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('patch_hmac_key must be overridden.')

  def compose_objects(self,
                      source_resources,
                      destination_resource,
                      request_config,
                      original_source_resource=None):
    """Concatenates a list of objects into a new object.

    Args:
      source_resources (list[ObjectResource|UnknownResource]): The objects to
        compose.
      destination_resource (resource_reference.UnknownResource): Metadata for
        the resulting composite object.
      request_config (RequestConfig): Object containing general API function
        arguments. Subclasses for specific cloud providers are available.
      original_source_resource (Resource|None): Useful for finding metadata to
        apply to final object. For instance, if doing a composite upload, this
        would represent the pre-split local file.

    Returns:
      resource_reference.ObjectResource with composite object's metadata.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('compose_object must be overridden.')

  def copy_object(self,
                  source_resource,
                  destination_resource,
                  request_config,
                  should_deep_copy_metadata=False,
                  progress_callback=None):
    """Copies an object within the cloud of one provider.

    Args:
      source_resource (resource_reference.ObjectResource): Resource for
        source object. Must have been confirmed to exist in the cloud.
      destination_resource (resource_reference.ObjectResource|UnknownResource):
        Resource for destination object. Existence doesn't have to be confirmed.
      request_config (RequestConfig): Object containing general API function
        arguments. Subclasses for specific cloud providers are available.
      should_deep_copy_metadata (bool): Rather than copying select fields of
        the source metadata, if True, copy everything. The request_config data
        (containing user args) overrides the deep-copied data.
      progress_callback (function): Optional callback function for progress
        notifications. Receives calls with arguments (bytes_transferred,
        total_size).

    Returns:
      resource_reference.ObjectResource with new object's metadata.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('copy_object must be overridden')

  def delete_object(self, object_url, request_config):
    """Deletes an object.

    Args:
      object_url (storage_url.CloudUrl): Url of object to delete.
      request_config (RequestConfig): Object containing general API function
        arguments. Subclasses for specific cloud providers are available.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
          this interface.
    """
    raise NotImplementedError('delete_object must be overridden.')

  def download_object(self,
                      cloud_resource,
                      download_stream,
                      request_config,
                      digesters=None,
                      do_not_decompress=False,
                      download_strategy=DownloadStrategy.ONE_SHOT,
                      progress_callback=None,
                      start_byte=0,
                      end_byte=None):
    """Gets object data.

    Args:
      cloud_resource (resource_reference.ObjectResource): Contains metadata and
        information about object being downloaded.
      download_stream (stream): Stream to send the object data to.
      request_config (RequestConfig): Contains arguments for API calls.
      digesters (dict): Dict of {string : digester}, where string is the name of
        a hash algorithm, and digester is a validation digester object that
        update(bytes) and digest() using that algorithm. Implementation can set
        the digester value to None to indicate supports bytes were not
        successfully digested on-the-fly.
      do_not_decompress (bool): If true, gzipped objects will not be
        decompressed on-the-fly if supported by the API.
      download_strategy (DownloadStrategy): Cloud API download strategy to use
        for download.
      progress_callback (function): Optional callback function for progress
        notifications. Receives calls with arguments (bytes_transferred,
        total_size).
      start_byte (int): Starting point for download (for resumable downloads and
        range requests). Can be set to negative to request a range of bytes
        (python equivalent of [:-3]).
      end_byte (int): Ending byte number, inclusive, for download (for range
        requests). If None, download the rest of the object.

    Returns:
      server_encoding (str): Useful for determining what the server actually
        sent versus what object metadata claims.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('download_object must be overridden.')

  def get_object_iam_policy(self, bucket_name, object_name, generation=None):
    """Gets object IAM policy.

    Args:
      bucket_name (str): Name of the bucket.
      object_name (str): Name of the object.
      generation (str|None): Generation of object.

    Returns:
      Provider-specific data type. Currently, only available for GCS so returns
        Apitools messages.Policy object. If supported for
        more providers in the future, use a generic container.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('get_object_iam_policy must be overridden.')

  def get_object_metadata(self,
                          bucket_name,
                          object_name,
                          request_config=None,
                          generation=None,
                          fields_scope=None):
    """Gets object metadata.

    If decryption is supported by the implementing class, this function will
    read decryption keys from configuration and appropriately retry requests to
    encrypted objects with the correct key.

    Args:
      bucket_name (str): Bucket containing the object.
      object_name (str): Object name.
      request_config (RequestConfig): Contains API call arguments.
      generation (string): Generation of the object to retrieve.
      fields_scope (FieldsScope): Determines the fields and projection
        parameters of API call.

    Returns:
      resource_reference.ObjectResource with object metadata.

    Raises:
      CloudApiError: API returned an error.
      NotFoundError: Raised if object does not exist.
      NotImplementedError: This function was not implemented by a class using
        this interface.
      ValueError: Invalid fields_scope.
    """
    raise NotImplementedError('get_object_metadata must be overridden.')

  def list_objects(self,
                   bucket_name,
                   prefix=None,
                   delimiter=None,
                   all_versions=None,
                   fields_scope=None):
    """Lists objects (with metadata) and prefixes in a bucket.

    Args:
      bucket_name (str): Bucket containing the objects.
      prefix (str): Prefix for directory-like behavior.
      delimiter (str): Delimiter for directory-like behavior.
      all_versions (boolean): If true, list all object versions.
      fields_scope (FieldsScope): Determines the fields and projection
        parameters of API call.

    Yields:
      Iterator over resource_reference.ObjectResource objects.

    Raises:
      NotImplementedError: This function was not implemented by a class using
        this interface.
      ValueError: Invalid fields_scope.
    """
    raise NotImplementedError('list_objects must be overridden.')

  def patch_object_metadata(self,
                            bucket_name,
                            object_name,
                            object_resource,
                            request_config,
                            fields_scope=None,
                            generation=None):
    """Updates object metadata with patch semantics.

    Args:
      bucket_name (str): Bucket containing the object.
      object_name (str): Object name.
      object_resource (resource_reference.ObjectResource): Contains metadata
        that will be used to update cloud object. May have different name than
        object_name argument.
      request_config (RequestConfig): Object containing general API function
        arguments. Subclasses for specific cloud providers are available.
      fields_scope (FieldsScope): Determines the fields and projection
        parameters of API call.
      generation (string): Generation (or version) of the object to update.

    Returns:
      resource_reference.ObjectResource with patched object metadata.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
      ValueError: Invalid fields_scope.
    """
    raise NotImplementedError('patch_object_metadata must be overridden.')

  def set_object_iam_policy(self,
                            bucket_name,
                            object_name,
                            policy,
                            generation=None):
    """Sets object IAM policy.

    Args:
      bucket_name (str): Name of the bucket.
      object_name (str): Name of the object.
      policy (object): Provider-specific data type. Currently, only available
        for GCS so Apitools messages.Policy object. If supported for more
        providers in the future, use a generic container.
      generation (str|None): Generation of object.

    Returns:
      Provider-specific data type. Currently, only available for GCS so returns
        Apitools messages.Policy object. If supported for
        more providers in the future, use a generic container.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('set_bucket_iam_policy must be overridden.')

  def upload_object(self,
                    source_stream,
                    destination_resource,
                    request_config,
                    source_resource=None,
                    serialization_data=None,
                    tracker_callback=None,
                    upload_strategy=UploadStrategy.SIMPLE):
    """Uploads object data and metadata.

    Args:
      source_stream (stream): Seekable stream of object data.
      destination_resource (resource_reference.ObjectResource|UnknownResource):
        Contains the correct metadata to upload.
      request_config (RequestConfig): Object containing general API function
        arguments. Subclasses for specific cloud providers are available.
      source_resource (resource_reference.FileObjectResource|None):
        Contains the source StorageUrl. Can be None if source is pure stream.
      serialization_data (dict): API-specific data needed to resume an upload.
        Only used with UploadStrategy.RESUMABLE.
      tracker_callback (Callable[[dict], None]): Function that writes a tracker
        file with serialization data. Only used with UploadStrategy.RESUMABLE.
      upload_strategy (UploadStrategy): Strategy to use for this upload.

    Returns:
      resource_reference.ObjectResource with uploaded object's metadata.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
    """
    raise NotImplementedError('upload_object must be overridden.')

  def get_service_agent(self, project_id=None, project_number=None):
    """Returns the email address (str) used to identify the service agent.

    For some providers, the service agent is responsible for encrypting and
    decrypting objects using CMEKs. project_number is useful because it may be
    in bucket metadata when project ID is not.

    If neither project_id or project_number are available, uses the
      default project configured in gcloud.


    Args:
      project_id (str|None): Project to get service account for. Takes
        precedence over project_number.
      project_number (int|None): Project to get service account for.

    Returns:
      Email of service account (str).
    """
    raise NotImplementedError('get_service_agent must be overridden.')

  def create_notification_configuration(
      self,
      url,
      pubsub_topic,
      custom_attributes=None,
      event_types=None,
      object_name_prefix=None,
      payload_format=NotificationPayloadFormat.JSON):
    """Creates a new notification on a bucket with the specified parameters.

    Args:
      url (storage_url.CloudUrl): Bucket URL.
      pubsub_topic (str): Cloud Pub/Sub topic to publish to.
      custom_attributes (dict[str, str]|None): Dictionary of custom attributes
        to apply to all notifications sent by the new configuration.
      event_types (list[NotificationEventType]|None): Event type filters, e.g.
        'OBJECT_FINALIZE'.
      object_name_prefix (str|None): Filter on object name.
      payload_format (NotificationPayloadFormat): Format of body of
        notifications sent by the new configuration.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
      ValueError: Received a non-bucket URL.

    Returns:
      Apitools Notification object for the new notification configuration.
    """
    raise NotImplementedError(
        'create_notification_configuration must be overridden.')

  def delete_notification_configuration(self, url, notification_id):
    """Deletes a notification configuration on a bucket.

    Args:
      url (storage_url.CloudUrl): Bucket URL.
      notification_id (str): Name of the notification configuration.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
      ValueError: Received a non-bucket URL.
    """
    raise NotImplementedError(
        'delete_notification_configuration must be overridden.')

  def get_notification_configuration(self, url, notification_id):
    """Gets a notification configuration on a bucket.

    Args:
      url (storage_url.CloudUrl): Bucket URL.
      notification_id (str): Name of the notification configuration.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
      ValueError: Received a non-bucket URL.
    """
    raise NotImplementedError(
        'get_notification_configuration must be overridden.')

  def list_notification_configurations(self, url):
    """Lists notification configurations on a bucket.

    Args:
      url (storage_url.CloudUrl): Bucket URL.

    Raises:
      CloudApiError: API returned an error.
      NotImplementedError: This function was not implemented by a class using
        this interface.
      ValueError: Received a non-bucket URL.

    Yields:
      List of  apitools Notification objects.
    """
    raise NotImplementedError(
        'list_notification_configurations must be overridden.')



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Wraps a Cloud Run DomainMapping message for field access convenience."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.run import k8s_object


MAPPING_ALREADY_EXISTS_CONDITION_REASON = 'MappingAlreadyExists'


class DomainMapping(k8s_object.KubernetesObject):
  """Wraps a Cloud Run DomainMapping message.

  Setting properties on a DomainMapping (where possible) writes through to the
  nested Kubernetes-style fields.
  """

  API_CATEGORY = 'domains.cloudrun.com'
  KIND = 'DomainMapping'

  @property
  def route_name(self):
    return self.spec.routeName

  @route_name.setter
  def route_name(self, value):
    self._m.spec.routeName = value

  @property
  def force_override(self):
    return self.spec.forceOverride or False

  @force_override.setter
  def force_override(self, value):
    self.spec.forceOverride = value

  @property
  def records(self):
    return getattr(self._m.status, 'resourceRecords', None)



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for describing instance templates."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.instance_templates import flags


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
class DescribeGA(base.DescribeCommand):
  """Describe a virtual machine instance template."""
  support_region_flag = False

  @classmethod
  def Args(cls, parser):
    DescribeGA.InstanceTemplateArg = flags.MakeInstanceTemplateArg(
        include_regional=cls.support_region_flag)
    DescribeGA.InstanceTemplateArg.AddArgument(
        parser, operation_type='describe')

  @staticmethod
  def GetServiceClient(client, ref):
    if ref.Collection() == 'compute.instanceTemplates':
      return client.apitools_client.instanceTemplates
    else:
      return client.apitools_client.regionInstanceTemplates

  @staticmethod
  def GetRequestMessage(client, ref):
    if ref.Collection() == 'compute.instanceTemplates':
      return client.messages.ComputeInstanceTemplatesGetRequest
    else:
      return client.messages.ComputeRegionInstanceTemplatesGetRequest

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    instance_template_ref = DescribeGA.InstanceTemplateArg.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=compute_flags.GetDefaultScopeLister(client),
        default_scope=compute_scope.ScopeEnum.GLOBAL)

    service_client = self.GetServiceClient(client, instance_template_ref)
    request_message = self.GetRequestMessage(client, instance_template_ref)
    return client.MakeRequests([
        (service_client, 'Get',
         request_message(**instance_template_ref.AsDict()))
    ])[0]


DescribeGA.detailed_help = {
    'brief':
        'Describe a virtual machine instance template',
    'DESCRIPTION':
        """\
        *{command}* displays all data associated with a Google Compute
        Engine virtual machine instance template.
        """,
    'EXAMPLES':
        """\
        To describe the instance template named 'INSTANCE-TEMPLATE', run:

          $ {command} INSTANCE-TEMPLATE
        """
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class DescribeAlpha(DescribeGA):
  support_region_flag = True



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""'logging logs list' command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager

from googlecloudsdk.api_lib.logging import util
from googlecloudsdk.calliope import base
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


class List(base.ListCommand):
  """List your project's logs.

  Only logs that contain log entries are listed.

  ## EXAMPLES

  To list all logs in current project:

    $ {command}

  To list all logs for a view:

    $ {command} --bucket=[BUCKET_ID] --location=[LOCATION] --view=[VIEW_ID]
  """

  @staticmethod
  def Args(parser):
    base.PAGE_SIZE_FLAG.RemoveFromParser(parser)
    base.URI_FLAG.RemoveFromParser(parser)
    parser.display_info.AddFormat('table(.:label=NAME)')
    view_group = parser.add_argument_group(
        help='These arguments are used in conjunction with the parent to '
        'construct a view resource.')
    view_group.add_argument(
        '--location',
        required=True,
        metavar='LOCATION',
        help='Location of the log bucket.')
    view_group.add_argument(
        '--bucket',
        required=True,
        help='Id of the log bucket.')
    view_group.add_argument(
        '--view',
        required=True,
        help='Id of the view.')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      The list of logs.
    """
    project = properties.VALUES.core.project.Get(required=True)

    project_ref = resources.REGISTRY.Parse(
        project, collection='cloudresourcemanager.projects')
    parent = project_ref.RelativeName()

    if args.IsSpecified('view'):
      # We are replacing the parent with the resourceName path for a view
      # instead of populating the resourceNames field.
      # This is due to the parent being a legacy required field.
      parent = util.CreateResourceName(
          util.CreateResourceName(
              util.CreateResourceName(parent, 'locations', args.location),
              'buckets', args.bucket), 'views', args.view)

    request = util.GetMessages().LoggingProjectsLogsListRequest(parent=parent)

    result = list_pager.YieldFromList(
        util.GetClient().projects_logs,
        request,
        field='logNames',
        limit=args.limit,
        batch_size=None,
        batch_size_attribute='pageSize')

    return result
